@@ -1,9 +1,10 @@
 #!/usr/bin/python
 # -- Content-Encoding: UTF-8 --
+"""
+Aliases to ease access to jsonrpclib classes
 
-# Create a configuration instance
-from jsonrpclib.config import Config
-config = Config.instance()
+:license: Apache License 2.0
+"""
 
 # Easy access to utility methods and classes
 from jsonrpclib.jsonrpc import Server, ServerProxy
