@@ -2,14 +2,34 @@
 # -- Content-Encoding: UTF-8 --
 """
 The serialization module
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+   http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
+:license: Apache License 2.0
+:version: 0.1.6
 """
 
-__version__ = "0.1.5"
+# Module version
+__version_info__ = (0, 1, 6)
+__version__ = ".".join(str(x) for x in __version_info__)
+
+# Documentation strings format
+__docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
 # Local package
-from jsonrpclib import config
+import jsonrpclib.config
 import jsonrpclib.utils as utils
 
 # Standard library
@@ -19,7 +39,8 @@ import re
 # ------------------------------------------------------------------------------
 
 # Supported transmitted code
-supported_types = utils.iter_types + utils.primitive_types
+supported_types = (utils.DictType,) + utils.iterable_types \
+                  + utils.primitive_types
 
 # Regex of invalid module characters
 invalid_module_chars = r'[^a-zA-Z0-9\_\.]'
@@ -34,17 +55,19 @@ class TranslationError(Exception):
 
 # ------------------------------------------------------------------------------
 
-def dump(obj, serialize_method=None, ignore_attribute=None, ignore=[]):
+def dump(obj, serialize_method=None, ignore_attribute=None, ignore=[],
+         config=jsonrpclib.config.DEFAULT):
     """
     Transforms the given object into a JSON-RPC compliant form.
     Converts beans into dictionaries with a __jsonclass__ entry.
     Doesn't change primitive types.
-    
+
     :param obj: An object to convert
     :param serialize_method: Custom serialization method
     :param ignore_attribute: Name of the object attribute containing the names
                              of members to ignore
     :param ignore: A list of members to ignore
+    :param config: A JSONRPClib Config instance
     :return: A JSON-RPC compliant object
     """
     if not serialize_method:
@@ -59,28 +82,16 @@ def dump(obj, serialize_method=None, ignore_attribute=None, ignore=[]):
         return obj
 
     # Iterative
-    if isinstance(obj, utils.iter_types):
+    elif isinstance(obj, utils.iterable_types):
+        # List, set or tuple
+        return [dump(item, serialize_method, ignore_attribute, ignore)
+                for item in obj]
 
-        if isinstance(obj, utils.DictType):
-            # It's a dictionary
-            new_obj = {}
-            for key, value in obj.items():
-                new_obj[key] = dump(value, serialize_method,
-                                     ignore_attribute, ignore)
-            return new_obj
-
-        else:
-            # ... list or tuple
-            new_obj = []
-            for item in obj:
-                new_obj.append(dump(item, serialize_method,
-                                     ignore_attribute, ignore))
-
-            if isinstance(obj, utils.TupleType):
-                # Keep the "tuple" type
-                new_obj = tuple(new_obj)
-
-            return new_obj
+    elif isinstance(obj, utils.DictType):
+        # Dictionary
+        return dict((key, dump(value, serialize_method,
+                               ignore_attribute, ignore))
+                    for key, value in obj.items())
 
     # It's not a standard type, so it needs __jsonclass__
     module_name = inspect.getmodule(obj).__name__
@@ -119,20 +130,21 @@ def dump(obj, serialize_method=None, ignore_attribute=None, ignore=[]):
         return return_obj
 
 
-def load(obj):
+def load(obj, classes=None):
     """
     If 'obj' is a dictionary containing a __jsonclass__ entry, converts the
     dictionary item into a bean of this class.
-    
+
     :param obj: An object from a JSON-RPC dictionary
+    :param classes: A custom {name: class} dictionary
     :return: The loaded object
     """
     # Primitive
     if isinstance(obj, utils.primitive_types):
         return obj
 
-    # List
-    elif isinstance(obj, (utils.ListType, utils.TupleType)):
+    # List, set or tuple
+    elif isinstance(obj, utils.iterable_types):
         return_obj = [load(entry) for entry in obj]
         if isinstance(obj, utils.TupleType):
             return_obj = tuple(return_obj)
@@ -162,12 +174,13 @@ def load(obj):
     # Load the class
     json_module_parts = json_module_clean.split('.')
     json_class = None
-    if len(json_module_parts) == 1:
+    if classes and len(json_module_parts) == 1:
         # Local class name -- probably means it won't work
-        if json_module_parts[0] not in config.classes.keys():
+        try:
+            json_class = classes[json_module_parts[0]]
+        except KeyError:
             raise TranslationError('Unknown class or module {0}.' \
                                    .format(json_module_parts[0]))
-        json_class = config.classes[json_module_parts[0]]
 
     else:
         # Module + class
@@ -191,19 +204,30 @@ def load(obj):
     # Create the object
     new_obj = None
     if isinstance(params, utils.ListType):
-        new_obj = json_class(*params)
+        try:
+            new_obj = json_class(*params)
 
-    if isinstance(params, utils.DictType):
-        new_obj = json_class(**params)
+        except TypeError as ex:
+            raise TranslationError("Error instantiating {0}: {1}"\
+                                   .format(json_class.__name__, ex))
+
+    elif isinstance(params, utils.DictType):
+        try:
+            new_obj = json_class(**params)
+
+        except TypeError as ex:
+            raise TranslationError("Error instantiating {0}: {1}"\
+                                   .format(json_class.__name__, ex))
 
     else:
-        raise TranslationError('Constructor args must be a dict or list.')
+        raise TranslationError("Constructor args must be a dict or a list, "
+                               "not {0}".format(type(params).__name__))
+
+    # Remove the class information, as it must be ignored during the
+    # reconstruction of the object
+    del obj['__jsonclass__']
 
     for key, value in obj.items():
-        if key == '__jsonclass__':
-            # Ignore the __jsonclass__ member
-            continue
-
         setattr(new_obj, key, value)
 
     return new_obj
