@@ -45,15 +45,23 @@ appropriately.
 >>> batch()
 [53, 5]
 
-See http://code.google.com/p/jsonrpclib/ for more info.
+See https://github.com/tcalmant/jsonrpclib for more info.
+
+:license: Apache License 2.0
+:version: 0.1.6
 """
 
-__version__ = "0.1.5"
+# Module version
+__version_info__ = (0, 1, 6)
+__version__ = ".".join(str(x) for x in __version_info__)
+
+# Documentation strings format
+__docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
 # Library includes
-from jsonrpclib import config
+import jsonrpclib.config
 import jsonrpclib.utils as utils
 
 # Standard library
@@ -61,16 +69,7 @@ import contextlib
 import sys
 import uuid
 
-if sys.version_info[0] < 3:
-    # Python 2
-    from urllib import splittype
-    from urllib import splithost
-    from xmlrpclib import Transport as XMLTransport
-    from xmlrpclib import SafeTransport as XMLSafeTransport
-    from xmlrpclib import ServerProxy as XMLServerProxy
-    from xmlrpclib import _Method as XML_Method
-
-else:
+try:
     # Python 3
     from urllib.parse import splittype
     from urllib.parse import splithost
@@ -79,6 +78,15 @@ else:
     from xmlrpc.client import ServerProxy as XMLServerProxy
     from xmlrpc.client import _Method as XML_Method
 
+except ImportError:
+    # Python 2
+    from urllib import splittype
+    from urllib import splithost
+    from xmlrpclib import Transport as XMLTransport
+    from xmlrpclib import SafeTransport as XMLSafeTransport
+    from xmlrpclib import ServerProxy as XMLServerProxy
+    from xmlrpclib import _Method as XML_Method
+
 # ------------------------------------------------------------------------------
 # JSON library import
 
@@ -128,7 +136,7 @@ except ImportError:
 class ProtocolError(Exception):
     """
     JSON-RPC error
-    
+
     ProtocolError[0] can be:
     * an error message (string)
     * a (code, message) tuple
@@ -138,7 +146,7 @@ class ProtocolError(Exception):
 class AppError(ProtocolError):
     """
     Application error: the error code is not in the pre-defined ones
-    
+
     AppError[0][0]: Error code
     AppError[0][1]: Error message or trace
     AppError[0][2]: Associated data
@@ -146,7 +154,7 @@ class AppError(ProtocolError):
     def data(self):
         """
         Retrieves the value found in the 'data' entry of the error, or None
-        
+
         :return: The data associated to the error, or None
         """
         return self[0][2]
@@ -154,7 +162,6 @@ class AppError(ProtocolError):
 
 class TransportMixIn(object):
     """ Just extends the XMLRPC transport where necessary. """
-    user_agent = config.user_agent
     # for Python 2.7 support
     _connection = None
 
@@ -165,19 +172,31 @@ class TransportMixIn(object):
     # Use the configuration to change the content-type
     readonly_headers = ('content-length', 'content-type')
 
+    def __init__(self, config=jsonrpclib.config.DEFAULT):
+        """
+        Sets up the transport
+
+        :param config: A JSONRPClib Config instance
+        """
+        # Store the configuration
+        self._config = config
+
+        # Set up the user agent
+        self.user_agent = config.user_agent
+
     def push_headers(self, headers):
         """
         Adds a dictionary of headers to the additional headers list
-        
+
         :param headers: A dictionary
         """
         self.additional_headers.append(headers)
 
     def pop_headers(self, headers):
         """
-        Removes the given dictionary from the additional headers list. 
+        Removes the given dictionary from the additional headers list.
         Also validates that given headers are on top of the stack
-        
+
         :param headers: Headers to remove
         :raise AssertionError: The given dictionary is not on the latest stored
                                in the additional headers list
@@ -189,7 +208,7 @@ class TransportMixIn(object):
     def emit_additional_headers(self, connection):
         """
         Puts headers as is in the request, filtered read only headers
-        
+
         :param connection: The request connection
         """
         additional_headers = {}
@@ -216,7 +235,7 @@ class TransportMixIn(object):
         request_body = utils.to_bytes(request_body)
 
         # "static" headers
-        connection.putheader("Content-Type", config.content_type)
+        connection.putheader("Content-Type", self._config.content_type)
         connection.putheader("Content-Length", str(len(request_body)))
 
         # Emit additional headers here in order not to override content-length
@@ -278,10 +297,11 @@ class ServerProxy(XMLServerProxy):
     """
 
     def __init__(self, uri, transport=None, encoding=None,
-                 verbose=0, version=None, headers=None, history=None):
+                 verbose=0, version=None, headers=None, history=None,
+                 config=jsonrpclib.config.DEFAULT):
         """
         Sets up the server proxy
-        
+
         :param uri: Request URI
         :param transport: Custom transport handler
         :param encoding: Specified encoding
@@ -289,7 +309,11 @@ class ServerProxy(XMLServerProxy):
         :param version: JSON-RPC specification version
         :param headers: Custom additional headers for each request
         :param history: History object (for tests)
+        :param config: A JSONRPClib Config instance
         """
+        # Store the configuration
+        self._config = config
+
         if not version:
             version = config.version
         self.__version = version
@@ -305,9 +329,9 @@ class ServerProxy(XMLServerProxy):
 
         if transport is None:
             if schema == 'https':
-                transport = SafeTransport()
+                transport = SafeTransport(config=config)
             else:
-                transport = Transport()
+                transport = Transport(config=config)
         self.__transport = transport
 
         self.__encoding = encoding
@@ -319,14 +343,16 @@ class ServerProxy(XMLServerProxy):
 
     def _request(self, methodname, params, rpcid=None):
         request = dumps(params, methodname, encoding=self.__encoding,
-                        rpcid=rpcid, version=self.__version)
+                        rpcid=rpcid, version=self.__version,
+                        config=self._config)
         response = self._run_request(request)
         check_for_errors(response)
         return response['result']
 
     def _request_notify(self, methodname, params, rpcid=None):
         request = dumps(params, methodname, encoding=self.__encoding,
-                        rpcid=rpcid, version=self.__version, notify=True)
+                        rpcid=rpcid, version=self.__version, notify=True,
+                        config=self._config)
         response = self._run_request(request, notify=True)
         check_for_errors(response)
         return
@@ -353,7 +379,7 @@ class ServerProxy(XMLServerProxy):
 
         if not response:
             return None
-        return_obj = loads(response)
+        return_obj = loads(response, self._config)
         return return_obj
 
     def __getattr__(self, name):
@@ -415,10 +441,11 @@ class _Notify(object):
 
 class MultiCallMethod(object):
 
-    def __init__(self, method, notify=False):
+    def __init__(self, method, notify=False, config=jsonrpclib.config.DEFAULT):
         self.method = method
         self.params = []
         self.notify = notify
+        self._config = config
 
     def __call__(self, *args, **kwargs):
         if len(kwargs) > 0 and len(args) > 0:
@@ -431,7 +458,8 @@ class MultiCallMethod(object):
 
     def request(self, encoding=None, rpcid=None):
         return dumps(self.params, self.method, version=2.0,
-                     encoding=encoding, rpcid=rpcid, notify=self.notify)
+                     encoding=encoding, rpcid=rpcid, notify=self.notify,
+                     config=self._config)
 
     def __repr__(self):
         return '%s' % self.request()
@@ -443,11 +471,12 @@ class MultiCallMethod(object):
 
 class MultiCallNotify(object):
 
-    def __init__(self, multicall):
+    def __init__(self, multicall, config=jsonrpclib.config.DEFAULT):
         self.multicall = multicall
+        self._config = config
 
     def __getattr__(self, name):
-        new_job = MultiCallMethod(name, notify=True)
+        new_job = MultiCallMethod(name, notify=True, config=self._config)
         self.multicall._job_list.append(new_job)
         return new_job
 
@@ -471,9 +500,10 @@ class MultiCallIterator(object):
 
 class MultiCall(object):
 
-    def __init__(self, server):
+    def __init__(self, server, config=jsonrpclib.config.DEFAULT):
         self._server = server
         self._job_list = []
+        self._config = config
 
     def _request(self):
         if len(self._job_list) < 1:
@@ -489,10 +519,10 @@ class MultiCall(object):
 
     @property
     def _notify(self):
-        return MultiCallNotify(self)
+        return MultiCallNotify(self, self._config)
 
     def __getattr__(self, name):
-        new_job = MultiCallMethod(name)
+        new_job = MultiCallMethod(name, config=self._config)
         self._job_list.append(new_job)
         return new_job
 
@@ -508,22 +538,25 @@ class Fault(object):
     """
     JSON-RPC error class
     """
-    def __init__(self, code=-32000, message='Server error', rpcid=None):
+    def __init__(self, code=-32000, message='Server error', rpcid=None,
+                 config=jsonrpclib.config.DEFAULT):
         """
         Sets up the error description
-        
+
         :param code: Fault code
         :param message: Associated message
         :param rpcid: Request ID
+        :param config: A JSONRPClib Config instance
         """
         self.faultCode = code
         self.faultString = message
         self.rpcid = rpcid
+        self.config = config
 
     def error(self):
         """
         Returns the error as a dictionary
-        
+
         :returns: A {'code', 'message'} dictionary
         """
         return {'code':self.faultCode, 'message':self.faultString}
@@ -531,36 +564,36 @@ class Fault(object):
     def response(self, rpcid=None, version=None):
         """
         Returns the error as a JSON-RPC response string
-        
+
         :param rpcid: Forced request ID
         :param version: JSON-RPC version
         :return: A JSON-RPC response string
         """
         if not version:
-            version = config.version
+            version = self.config.version
 
         if rpcid:
             self.rpcid = rpcid
 
         return dumps(self, methodresponse=True, rpcid=self.rpcid,
-                     version=version)
+                     version=version, config=self.config)
 
     def dump(self, rpcid=None, version=None):
         """
         Returns the error as a JSON-RPC response dictionary
-        
+
         :param rpcid: Forced request ID
         :param version: JSON-RPC version
         :return: A JSON-RPC response dictionary
         """
         if not version:
-            version = config.version
+            version = self.config.version
 
         if rpcid:
             self.rpcid = rpcid
 
         return dump(self, is_response=True, rpcid=self.rpcid,
-                    version=version)
+                    version=version, config=self.config)
 
     def __repr__(self):
         """
@@ -573,12 +606,14 @@ class Payload(object):
     """
     JSON-RPC content handler
     """
-    def __init__(self, rpcid=None, version=None):
+    def __init__(self, rpcid=None, version=None,
+                 config=jsonrpclib.config.DEFAULT):
         """
         Sets up the JSON-RPC handler
-        
+
         :param rpcid: Request ID
         :param version: JSON-RPC version
+        :param config: A JSONRPClib Config instance
         """
         if not version:
             version = config.version
@@ -590,7 +625,7 @@ class Payload(object):
     def request(self, method, params=[]):
         """
         Prepares a method call request
-        
+
         :param method: Method name
         :param params: Method parameters
         :return: A JSON-RPC request dictionary
@@ -615,7 +650,7 @@ class Payload(object):
     def notify(self, method, params=[]):
         """
         Prepares a notification request
-        
+
         :param method: Notification name
         :param params: Notification parameters
         :return: A JSON-RPC notification dictionary
@@ -635,7 +670,7 @@ class Payload(object):
     def response(self, result=None):
         """
         Prepares a response dictionary
-        
+
         :param result: The result of method call
         :return: A JSON-RPC response dictionary
         """
@@ -652,7 +687,7 @@ class Payload(object):
     def error(self, code=-32000, message='Server error.'):
         """
         Prepares an error dictionary
-        
+
         :param code: Error code
         :param message: Error message
         :return: A JSON-RPC error dictionary
@@ -668,16 +703,17 @@ class Payload(object):
 # ------------------------------------------------------------------------------
 
 def dump(params=[], methodname=None, rpcid=None, version=None,
-         is_response=None, is_notify=None):
+         is_response=None, is_notify=None, config=jsonrpclib.config.DEFAULT):
     """
     Prepares a JSON-RPC dictionary (request, notification, response or error)
-    
+
     :param params: Method parameters (if a method name is given) or a Fault
     :param methodname: Method name
     :param rpcid: Request ID
     :param version: JSON-RPC version
     :param is_response: If True, this is a response dictionary
     :param is_notify: If True, this is a notification request
+    :param config: A JSONRPClib Config instance
     :return: A JSON-RPC dictionary
     """
     # Default version
@@ -727,10 +763,11 @@ def dump(params=[], methodname=None, rpcid=None, version=None,
 
 
 def dumps(params=[], methodname=None, methodresponse=None,
-          encoding=None, rpcid=None, version=None, notify=None):
+          encoding=None, rpcid=None, version=None, notify=None,
+          config=jsonrpclib.config.DEFAULT):
     """
     Prepares a JSON-RPC request/response string
-    
+
     :param params: Method parameters (if a method name is given) or a Fault
     :param methodname: Method name
     :param methodresponse: If True, this is a response dictionary
@@ -738,10 +775,12 @@ def dumps(params=[], methodname=None, methodresponse=None,
     :param rpcid: Request ID
     :param version: JSON-RPC version
     :param notify: If True, this is a notification request
+    :param config: A JSONRPClib Config instance
     :return: A JSON-RPC dictionary
     """
     # Prepare the dictionary
-    request = dump(params, methodname, rpcid, version, methodresponse, notify)
+    request = dump(params, methodname, rpcid, version, methodresponse, notify,
+                   config)
 
     # Set the default encoding
     if not encoding:
@@ -751,11 +790,12 @@ def dumps(params=[], methodname=None, methodresponse=None,
     return jdumps(request, encoding=encoding)
 
 
-def load(data):
+def load(data, config=jsonrpclib.config.DEFAULT):
     """
     Loads a JSON-RPC request/response dictionary. Calls jsonclass to load beans
-    
+
     :param data: A JSON-RPC dictionary
+    :param config: A JSONRPClib Config instance (or None for default values)
     :return: A parsed dictionary or None
     """
     if data is None:
@@ -767,16 +807,17 @@ def load(data):
     # { 'jsonrpc':'2.0', 'error': fault.error(), id: None }
     if config.use_jsonclass:
         # Convert beans
-        data = jsonclass.load(data)
+        data = jsonclass.load(data, config.classes)
 
     return data
 
 
-def loads(data):
+def loads(data, config=jsonrpclib.config.DEFAULT):
     """
     Loads a JSON-RPC request/response string. Calls jsonclass to load beans
-    
+
     :param data: A JSON-RPC string
+    :param config: A JSONRPClib Config instance (or None for default values)
     :return: A parsed dictionary or None
     """
     if data == '':
@@ -787,14 +828,14 @@ def loads(data):
     result = jloads(data)
 
     # Load the beans
-    return load(result)
+    return load(result, config)
 
 # ------------------------------------------------------------------------------
 
 def check_for_errors(result):
     """
     Checks if a result dictionary signals an error
-    
+
     :param result: A result dictionary
     :raise TypeError: Invalid parameter
     :raise NotImplementedError: Unknown JSON-RPC version
@@ -876,7 +917,7 @@ def isbatch(result):
 def isnotification(request):
     """
     Tests if the given request is a notification
-    
+
     :param request: A request dictionary
     :return: True if the request is a notification
     """
